package pages

import (
	"fmt"
	"git.binarythought.com/cdramey/qurl/assets"
	"git.binarythought.com/cdramey/qurl/obj"
	"git.binarythought.com/cdramey/qurl/storage"
	"html/template"
	"net/http"
)

type RootHandler struct {
	Storage storage.Storage

	index  *assets.StaticContent
	css    *assets.StaticContent
	favi   *assets.StaticContent
	usage  *assets.StaticContent
	submit *template.Template
}

func (ctx *RootHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	fname := r.URL.Path

	switch fname {
	case "/index.html":
		fallthrough
	case "/":
		ctx.index.ServeHTTP(w, r)

	case "/submit.html":
		ctx.ServeSubmit(w, r)

	case "/qurl.css":
		ctx.css.ServeHTTP(w, r)

	case "/favicon.ico":
		ctx.favi.ServeHTTP(w, r)

	case "/api/usage.html":
		ctx.usage.ServeHTTP(w, r)

	case "/api/url":
		ctx.ServeAPI(w, r)

	default:
		id, err := obj.FromString(fname[1:len(fname)])
		if err != nil {
			http.Redirect(w, r, "/", http.StatusTemporaryRedirect)
			return
		}

		q, err := ctx.Storage.GetQURLByID(id)
		if err != nil {
			http.Error(w, fmt.Sprintf("Database error: %s", err.Error()),
				http.StatusInternalServerError)
			return
		}

		if q == nil {
			http.Redirect(w, r, "/", http.StatusTemporaryRedirect)
			return
		}

		http.Redirect(w, r, q.URL, http.StatusPermanentRedirect)
	}
}

func (ctx *RootHandler) Init() error {
	var err error

	// Initialize the static content object for the index page
	ctx.index, err = assets.NewStaticContent("index.html", "text/html")
	if err != nil {
		return err
	}

	// Initialize the static content object for the css
	ctx.css, err = assets.NewStaticContent("qurl.css", "text/css")
	if err != nil {
		return err
	}

	// Initialize the static content object favicon
	ctx.favi, err = assets.NewStaticContent("favicon.ico", "image/x-icon")
	if err != nil {
		return err
	}

	ctx.usage, err = assets.NewStaticContent("usage.html", "text/html")
	if err != nil {
		return err
	}

	// Initialize submit page template
	data, err := assets.ReadFile("submit.html")
	if err != nil {
		return err
	}
	ctx.submit = template.New("submit.html")
	_, err = ctx.submit.Parse(string(data))
	if err != nil {
		return err
	}
	return nil
}
